@@ -76,21 +76,23 @@ class GAlearner:
                  latent_dim=512,
                  sparsity=0.1,
                  hof_size=3,
+                 initial_separate_spaces = True,
                  scoring_metric=None,
                  top_k_importances=25,
                  representation_type="neurosymbolic",
                  binarize_importances=False,
                  memory_storage="memory",
                  classifier=None,
-                 n_fold_cv=5,
+                 n_fold_cv=4,
                  classifier_hyperparameters=None,
                  custom_transformer_pipeline=None,
                  combine_with_existing_representation = False,
                  conceptnet_url = "https://s3.amazonaws.com/conceptnet/downloads/2019/edges/conceptnet-assertions-5.7.0.csv.gz",
                  default_importance = 0.05,
+                 classifier_preset = "default",
                  verbose = 1):
-        """
-        The object initialization method
+        
+        """The object initialization method
 
         :param train_sequences_raw: a list of texts
         :param train_targets: a list of natural numbers (targets, multiclass)
@@ -100,6 +102,7 @@ class GAlearner:
         :param latent_dim: The latent dimension of embeddings
         :param sparsity: The assumed sparsity of the induced space (see paper)
         :param hof_size: Hof many final models to consider?
+        :param initial_separate_spaces: Whether to include separate spaces as part of the initial population.
         :param scoring_metric: The type of metric to optimize (sklearn-compatible)
         :param top_k_importances: How many top importances to remember for explanations.
         :param representation_type: symbolic, neurosymbolic or custom
@@ -108,7 +111,9 @@ class GAlearner:
         :param classifier: custom classifier. If none, linear learners are used.
         :param classifier_hyperparameters: The space to be optimized w.r.t. the classifier param.
         :param conceptnet_url: URL of the conceptnet used.
+        :param classifier_preset: Type of classification to be considered (default = paper), mini -> very lightweight regression, emphasis on space exploration.
         :param default_importance: Minimum possible initial weight.
+
         """
         
         logo = """
@@ -150,6 +155,7 @@ class GAlearner:
         self.verbose = verbose
         if self.verbose: print(logo)
         self.default_importance = default_importance
+        self.classifier_preset = classifier_preset
         
         if self.verbose: logging.info("Instantiated the evolution-based learner.")
         self.scoring_metric = scoring_metric
@@ -157,6 +163,7 @@ class GAlearner:
         self.representation_type = representation_type
         self.custom_transformer_pipeline = custom_transformer_pipeline
         self.combine_with_existing_representation = combine_with_existing_representation
+        self.initial_separate_spaces = initial_separate_spaces
 
         if not self.custom_transformer_pipeline is None:
             if self.verbose: logging.info("Using custom feature transformations.")
@@ -429,6 +436,14 @@ class GAlearner:
             generic_individual = generic_individual * noise + self.default_importance
             ind[:] = np.abs(generic_individual)
 
+        ## Separate spaces -- each subspace massively amplified
+        if self.initial_separate_spaces:
+            for k in range(self.weight_params):
+                individual = self.population[0]
+                individual[:] = np.zeros(self.weight_params)
+                individual[k] = 1 ## amplify particular subspace.
+                self.population.append(individual)
+
     def apply_weights(self,
                       parameters,
                       custom_feature_space=False,
@@ -490,12 +505,27 @@ class GAlearner:
 
             else:
                 ## this is for screening purposes.
-                parameters = {
-                    "loss": ["hinge", "log"],
-                    "penalty": ["elasticnet"],
-                    "alpha": [0.01, 0.001, 0.0001],
-                    "l1_ratio": [0, 0.1, 0.5, 0.9]
-                }
+
+                if self.classifier_preset == "default":
+                    parameters = {
+                        "loss": ["hinge", "log"],
+                        "penalty": ["elasticnet"],
+                        "alpha": [0.01, 0.001, 0.0001],
+                        "l1_ratio": [0, 0.1, 0.5, 0.9]
+                    }
+                    
+                elif self.classifier_preset == "mini-l1":
+                     parameters = {
+                        "loss": ["log"],
+                        "penalty": ["l1"]
+                    }
+
+                elif self.classifier_preset == "mini-l2":
+                     parameters = {
+                        "loss": ["log"],
+                        "penalty": ["l2"]
+                    }
+                    
         else:
 
             parameters = self.classifier_hyperparameters
@@ -513,7 +543,7 @@ class GAlearner:
                                parameters,
                                verbose=self.verbose,
                                n_jobs=self.num_cpu,
-                               cv=10,
+                               cv=self.n_fold_cv,
                                scoring=performance_score,
                                refit=True)
 
@@ -1055,7 +1085,7 @@ class GAlearner:
                 cf1 = f1
 
             self.population = self.toolbox.select(self.population + offspring,
-                                                  k=len(self.population),
+                                                  k=nind,
                                                   tournsize=int(nind / 3))
 
         try:
